@@ -16,18 +16,22 @@
  * index		strchr
  * rindex		strrchr
  *
- * Also, the code for suspend is only given for BSD
  */
 
+/* define one of the following */
 /* #define BSD		/* define for 4.n BSD  */
-			/* also for relatives like SunOS */
+			/* also for relatives like SunOS and DG/UX */
 #define ULTRIX		/* define for Ultrix v3.0 or higher (but not lower) */
 			/* Use BSD for < v3.0 */
 			/* "ULTRIX" not to be confused with "ultrix" */
 /* #define SYSV		/* define for System V */
+/* #define HPUX		/* Hewlett-Packard's Unix, version 6.5 or higher */
+			/* use SYSV for < v6.5 */
+
+
+/* define any of the following that are appropriate */
 /* #define NETWORK	/* if running on a networked system */
 			/* e.g. Suns sharing a playground through NFS */
-
 /* #define GENIX	/* Yet Another Unix Clone */
 /* #define HISX		/* Bull Unix for XPS Machines */
 /* #define UNIXPC	/* use in addition to SYSV for AT&T 7300/3B1 */
@@ -36,6 +40,7 @@
 
 /* #define PYRAMID_BUG 	/* avoid a bug on the Pyramid */
 /* #define APOLLO	/* same for the Apollo */
+/* #define BSD_43_BUG	/* for real 4.3BSD cc's without schain botch fix */
 /* #define MICROPORT_BUG /* problems with large arrays in structs */
 /* #define MICROPORT_286_BUG /* Changes needed in termcap.c to get it to
 			   run with Microport Sys V/AT version 2.4.
@@ -66,34 +71,63 @@
  * #define DEF_PAGER	".../mydir/mypager"
  */
 
+
+
 /*
  * If you define MAIL, then the player will be notified of new mail
- * when it arrives. If you also define DEF_MAILREADER then this will
+ * when it arrives.  If you also define DEF_MAILREADER then this will
  * be the default mail reader, and can be overridden by the environment
  * variable MAILREADER; otherwise an internal pager will be used.
  * A stat system call is done on the mailbox every MAILCKFREQ moves.
  */
 
-#define	MAIL		/* Deliver mail during the game */
-#ifdef MAIL
+#define MAIL			/* Deliver mail during the game */
 
+/* The Andrew Message System does mail a little differently from normal
+ * UNIX.  Mail is deposited in the user's own directory in ~/Mailbox 
+ * (another directory).  MAILBOX is the element that will be added on to 
+ * the user's home directory path to generate the Mailbox path - just in 
+ * case other Andrew sites do it differently from CMU.
+ *
+ *		dan lovinger
+ *		dl2n+@andrew.cmu.edu (dec 19 1989)
+ */
+
+/* #define AMS			/* use Andrew message system for mail */
+
+/* NO_MAILREADER is for kerberos authentcating filesystems where it is
+ * essentially impossible to securely exec child proccesses, like mail
+ * readers, when the game is running under a special token.
+ *
+ *             dan
+ */
+
+/* #define NO_MAILREADER 	/* have mail daemon just tell player of mail */
+
+#ifdef	MAIL
 # if defined(BSD) || defined(ULTRIX)
-#define	DEF_MAILREADER	"/usr/ucb/Mail"
-# else
-#  if defined(SYSV) || defined(DGUX)
-#   ifdef M_XENIX
-#define	DEF_MAILREADER	"/usr/bin/mail"
-#   else
-#define	DEF_MAILREADER	"/usr/bin/mailx"
-#   endif
+#  ifdef AMS
+#define AMS_MAILBOX	"/Mailbox"
 #  else
-#define	DEF_MAILREADER	"/bin/mail"
+#define	DEF_MAILREADER	"/usr/ucb/Mail"
 #  endif
+#else
+# if defined(SYSV) || defined(DGUX) || defined(HPUX)
+#  ifdef M_XENIX
+#define	DEF_MAILREADER	"/usr/bin/mail"
+#  else
+#define	DEF_MAILREADER	"/usr/bin/mailx"
+#  endif
+# else
+#define	DEF_MAILREADER	"/bin/mail"
 # endif
+#endif
 
 #define	MAILCKFREQ	50
 #endif	/* MAIL */
 
+
+
 #ifdef COMPRESS
 /* Some implementations of compress need a 'quiet' option.
  * If you've got one of these versions, put -q here.
@@ -110,7 +144,18 @@
  * The remainder of the file should not need to be changed.
  */
 
-#if (defined(BSD) || defined(ULTRIX)) && !defined(MSDOS)
+/*
+ * BSD/ULTRIX systems are normally the only ones that can suspend processes.
+ * Suspending NetHack processes cleanly should be easy to add to other systems
+ * that have SIGTSTP.  Currently the only such system known to work is HPUX;
+ * other systems will probably require tweaks to unixtty.c and ioctl.c.
+ */
+#if defined(BSD) || defined(ULTRIX) || defined (HPUX)
+#define	SUSPEND		/* let ^Z suspend the game */
+#endif
+
+
+#if defined(BSD) || defined(ULTRIX)
 #include	<sys/time.h>
 #else
 #include	<time.h>
@@ -130,9 +175,10 @@
 #include "system.h"
 
 #if defined(BSD) || defined(ULTRIX)
-#define	SUSPEND		/* let ^Z suspend the game */
+# ifndef DGUX
 #define memcpy(d, s, n)		bcopy(s, d, n)
 #define memcmp(s1, s2, n)	bcmp(s2, s1, n)
+# endif
 #else	/* therefore SYSV */
 #define index	strchr
 #define rindex	strrchr
