@@ -11,21 +11,25 @@
 #define MONFLAG_H
 #include "hack.h"
 
-#if defined(BSD) || defined(ULTRIX)
+#if defined(BSD) || defined(ULTRIX) || defined(HPUX)
+# ifdef HPUX
+#include	<bsdtty.h>
+# else
 #include	<sgtty.h>
+# endif
 struct ltchars ltchars;
 struct ltchars ltchars0 = { -1, -1, -1, -1, -1, -1 }; /* turn all off */
 #else
 #include	<termio.h>	/* also includes part of <sgtty.h> */
 struct termio termio;
-#ifdef AMIX
+# ifdef AMIX
 #include <sys/ioctl.h>
-#endif /* AMIX */
+# endif /* AMIX */
 #endif
 
 void
 getioctls() {
-#if defined(BSD) || defined(ULTRIX)
+#if defined(BSD) || defined(ULTRIX) || defined(HPUX)
 	(void) ioctl(fileno(stdin), (int) TIOCGLTC, (char *) &ltchars);
 	(void) ioctl(fileno(stdin), (int) TIOCSLTC, (char *) &ltchars0);
 #else
@@ -56,7 +60,7 @@ getioctls() {
 
 void
 setioctls() {
-#if defined(BSD) || defined(ULTRIX)
+#if defined(BSD) || defined(ULTRIX) || defined(HPUX)
 	(void) ioctl(fileno(stdin), (int) TIOCSLTC, (char *) &ltchars);
 #else
 	/* Now modified to run under Sys V R3.	- may have to be #ifdef'ed */
