@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)pager.c	3.0	89/11/15
+/*	SCCS Id: @(#)pager.c	3.0	89/11/19
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -6,10 +6,11 @@
 /* Also readmail() and doshell(), and generally the things that
    contact the outside world. */
 
+#define MONATTK_H	/* comment line for pre-compiled headers */
 /* block some unused #defines to avoid overloading some cpp's */
-#define MONATTK_H
 #include	"hack.h"
 
+#include <ctype.h>
 #ifndef NO_SIGNAL
 #include <signal.h>
 #endif
@@ -21,11 +22,56 @@ extern WindowPtr	HackWindow;
 extern short macflags;
 #endif
 
-static char hc = 0;
+#ifndef SEEK_SET
+#define SEEK_SET 0
+#endif
+
+#ifndef OVLB
+OSTATIC char hc;
+#else /* OVLB */
+XSTATIC char hc = 0;
+#endif /* OVLB */
 
 static void FDECL(page_more, (FILE *,int));
-static boolean FDECL(clear_help, (CHAR_P));
-static boolean FDECL(valid_help, (CHAR_P));
+OSTATIC boolean FDECL(clear_help, (CHAR_P));
+OSTATIC boolean FDECL(valid_help, (CHAR_P));
+static boolean FDECL(pmatch,(const char *,const char *));
+static boolean FDECL(outspec,(const char *,int));
+static const char *FDECL(lookat,(int,int,UCHAR_P));
+#ifdef WIZARD
+static void NDECL(wiz_help);
+#endif
+static void NDECL(help_menu);
+
+#ifdef OVLB
+
+/*
+ * simple pattern matcher: '*' matches 0 or more characters
+ * returns TRUE if strng matches patrn
+ */
+
+static boolean
+pmatch(patrn, strng)
+	const char *patrn, *strng;
+{
+	char s, p;
+
+	s = *strng;
+	p = *patrn;
+
+	if (!p) {
+		return (s == 0);
+	}
+
+	if (p == '*') {
+		if (!patrn[1] || pmatch(patrn+1, strng)) {
+			return TRUE;
+		}
+		return (s ? pmatch(patrn, strng+1) : FALSE);
+	}
+
+	return (p == s) ? pmatch(patrn+1, strng+1) : FALSE;
+}
 
 /*
  * print out another possibility for dowhatis. "new" is the possible new
@@ -36,7 +82,7 @@ static boolean FDECL(valid_help, (CHAR_P));
 
 static boolean
 outspec(new, out_flag)
-char *new;
+const char *new;
 int out_flag;
 {
 	static char old[50];
@@ -45,25 +91,178 @@ int out_flag;
 		return FALSE;		/* don't print the same thing twice */
 
 	if (out_flag)
-		pline("(or %s)", new);
+		pline("(or %s)", an(new));
 
 	Strcpy(old, new);
 	return 1;
 }
 
+/*
+ * return the name of the character ch found at (x,y)
+ */
+
+static
+const char *
+lookat(x, y, ch)
+int x,y;
+uchar ch;
+{
+	register struct monst *mtmp;
+	register struct obj *otmp;
+	struct trap *trap;
+	static char answer[50];
+	register char *s, *t;
+	uchar typ;
+
+	answer[0] = 0;
+
+	if(MON_AT(x,y)) {
+		mtmp = m_at(x,y);
+		if (!showmon(mtmp) || Hallucination)
+			mtmp = (struct monst *)0;
+	} else
+		mtmp = (struct monst *) 0;
+	typ = levl[x][y].typ;
+	if (!Invisible 
+#ifdef POLYSELF
+			&& !u.uundetected
+#endif
+			&& u.ux==x && u.uy==y) {
+		Sprintf(answer, "%s named %s",
+#ifdef POLYSELF
+			u.mtimedone ? mons[u.umonnum].mname :
+#endif
+			pl_character, plname);
+	} else if (mtmp && !mtmp->mimic)
+		Sprintf(answer, "%s%s",
+		   mtmp->mtame ? "tame " :
+		   mtmp->mpeaceful ? "peaceful " : "",
+		   strncmp(lmonnam(mtmp), "the ", 4)
+			  ? lmonnam(mtmp) : lmonnam(mtmp)+4);
+	else if (!levl[x][y].seen)
+		Strcpy(answer,"dark part of a room");
+	else if (mtmp && mtmp->mimic) {
+		if (mtmp->m_ap_type == M_AP_FURNITURE) {
+			if (mtmp->mappearance == S_altar)
+				Strcpy(answer, "neutral altar");
+			else
+				Strcpy(answer, explainsyms[mtmp->mappearance]);
+		}
+		else if (mtmp->m_ap_type == M_AP_OBJECT) {
+			if (mtmp->mappearance == STRANGE_OBJECT)
+				Strcpy(answer, "strange object");
+			else {
+				otmp = mksobj((int) mtmp->mappearance,FALSE );
+				Strcpy(answer, distant_name(otmp, xname));
+				free((genericptr_t) otmp);
+			}
+		}
+		else if (mtmp->m_ap_type == M_AP_GOLD)
+			Strcpy(answer, "pile of gold");
+	}
+	else if (OBJ_AT(x, y)) {
+		otmp = level.objects[x][y];
+		Strcpy(answer, distant_name(otmp, xname));
+	}
+	else if (ch == GOLD_SYM) {
+		Strcpy(answer, "pile of gold");
+	}
+#ifdef ALTARS
+	else if (ch == ALTAR_SYM && IS_ALTAR(typ)) {
+		int kind = levl[x][y].altarmask & ~A_SHRINE;
+		Sprintf( answer, "%s altar",
+			(kind == A_CHAOS) ? "chaotic" :
+			(kind == A_NEUTRAL) ? "neutral" :
+			 "lawful" );
+	}
+#endif
+#ifdef STRONGHOLD
+	else if ((ch == DB_VWALL_SYM || ch == DB_HWALL_SYM) && is_db_wall(x,y))
+		Strcpy(answer,"raised drawbridge");
+#endif
+#ifdef THRONES
+	else if ((ch == THRONE_SYM) && IS_THRONE(typ))
+		Strcpy(answer, "throne");
+#endif
+	else if ( (ch==H_OPEN_DOOR_SYM ||
+		   ch==V_OPEN_DOOR_SYM ||
+		   ch==CLOSED_DOOR_SYM ||
+		   ch==NO_DOOR_SYM) &&
+		  IS_DOOR(typ) ) {
+		switch(levl[x][y].doormask & ~D_TRAPPED) {
+			case D_NODOOR: Strcpy(answer,"doorway"); break;
+			case D_BROKEN: Strcpy(answer,"broken door"); break;
+			case D_ISOPEN: Strcpy(answer,"open door"); break;
+			default:       Strcpy(answer,"closed door"); break;
+					   /* locked or not */
+		}
+	}
+#ifdef SINKS
+	else if (ch == SINK_SYM && IS_SINK(levl[x][y].typ))
+		Strcpy(answer,"sink");
+#endif
+	else if ((ch == TRAP_SYM || ch == WEB_SYM) && (trap = t_at(x, y))) {
+		if (trap->ttyp == WEB && ch == WEB_SYM)
+			Strcpy(answer, "web");
+		else if (trap->ttyp != MONST_TRAP && ch == TRAP_SYM) {
+			Strcpy(answer, traps[ Hallucination ?
+				rn2(TRAPNUM-3)+3 : trap->ttyp]);
+		/* strip leading garbage */
+			for (s = answer; *s && *s != ' '; s++) ;
+			if (*s) ++s;
+			for (t = answer; *t++ = *s++; ) ;
+		}
+	}
+	else if (ch == UP_SYM && x == xupstair && y == yupstair)
+		Strcpy(answer, "staircase up");
+	else if (ch == DN_SYM && x == xdnstair && y == ydnstair)
+		Strcpy(answer, "staircase down");
+#ifdef STRONGHOLD
+	else if (ch == UPLADDER_SYM && x && x == xupladder && y == ydnladder)
+		Strcpy(answer, "ladder up");
+	else if (ch == DNLADDER_SYM && x && x == xdnladder && y == ydnladder)
+		Strcpy(answer, "ladder down");
+#endif
+	else if (IS_ROOM(typ)) {
+		if (ch == ROOM_SYM) {
+			if (levl[x][y].icedpool)
+			    Strcpy(answer,"iced pool");
+			else
+			    Strcpy(answer,"floor of a room");
+		}
+		else if (ch == STONE_SYM || ch == ' ')
+			Strcpy(answer,"dark part of a room");
+	}
+	else if (ch == CORR_SYM && SPACE_POS(typ))
+		Strcpy(answer,"corridor");
+	else if (!ACCESSIBLE(typ)) {
+		if (ch == STONE_SYM || ch == ' ')
+			Strcpy(answer,"dark part of a room");
+		else
+			Strcpy(answer,"wall");
+	}
+	return answer;
+}
+
+	
 int
 dowhatis()
 {
 	FILE *fp;
-	char bufr[BUFSZ+6];
-	register char *buf = &bufr[6], *ep;
-	uchar q, typ;
+	char buf[BUFSZ], inpbuf[BUFSZ];
+	register char *ep, *inp = inpbuf;
+	char *alt = 0;		/* alternate description */
+#ifdef __GNULINT__
+	const char *firstmatch = 0;
+#else
+	const char *firstmatch;
+#endif
+	uchar q;
 	register int i;
 	coord	cc;
 	boolean oldverb = flags.verbose;
-	boolean found_in_file = FALSE;
+	boolean found_in_file = FALSE, need_to_print = FALSE;
 	int	found = 0;
-	register struct monst *mtmp;
 
 #ifdef OS2_CODEVIEW
 	char tmp[PATHLEN];
@@ -92,14 +291,24 @@ dowhatis()
 		return 0;
 	} else if (q == 'n') {
 		cc.x = cc.y = -1;
-		pline("Specify what? ");
-		q = readchar();
+		pline("Specify what? (type the word) ");
+		getlin(inp);
+		if (inp[0] == '\033' || !inp[0]) {
+			(void)fclose(fp);
+			return 0;
+		}
+		if (!inp[1])
+			q = inp[0];
+		else
+			q = 0;
 	} else {
 		cc.x = u.ux;
 		cc.y = u.uy;
 selobj:
-		found_in_file = FALSE;
+		need_to_print = found_in_file = FALSE;
 		found = 0;
+		inp = inpbuf;
+		alt = 0;
 		(void) outspec("", 0);		/* reset output */
 		if(flags.verbose)
 			pline("Please move the cursor to an unknown object.");
@@ -117,142 +326,161 @@ selobj:
 			q = ' ';
 	}
 
-	if (index(quitchars, (char)q)) {
+	if (!q)
+		goto checkfile; /* user typed in a complete string */
+
+	if (q != ' ' && index(quitchars, (char)q)) {
 		(void) fclose(fp); /* sweet@scubed */
 		flags.verbose = oldverb;
 		return 0;
 	}
+/*
+ * if the user just typed one letter, or we're identifying from the
+ * screen, then we have to check all the possibilities and print them
+ * out for him/her
+ */
 
-/* now check for symbols defined in the data file */
-	if(q != '\t')
-	while(fgets(buf,BUFSZ,fp)) {
-		if(*buf == q) {
-			ep = index(buf, '\n');
-			if(ep) *ep = 0;
-			/* else: bad data file */
-			/* Expand tab 'by hand' */
-			if (buf[1] == '\t') {
-				buf = bufr;
-				buf[0] = q;
-				(void) strncpy(buf+1, "       ", 7);
-			}
-			pline("%s", buf);	/* watch out for % in output */
-			(void) outspec("", 0);
+/* Check for monsters */
+	for (i = 0; monsyms[i]; i++) {
+		if (q == monsyms[i]) {
+			need_to_print = TRUE;
+			pline("%c       %s",q,an(monexplain[i]));
+			(void) outspec(firstmatch = monexplain[i], 0);
 			found++;
-			found_in_file = TRUE;
 			break;
 		}
 	}
 
+/* Now check for objects */
+	for (i = 0; objsyms[i]; i++) {
+		if (q == objsyms[i]) {
+			need_to_print = TRUE;
+			if (!found) {
+				pline("%c       %s",q,an(objexplain[i]));
+				(void)outspec(firstmatch = objexplain[i], 0);
+				found++;
+			}
+			else if (outspec(objexplain[i], 1))
+				found++;
+		}
+	}
+
 /* Now check for graphics symbols */
 	for (i = 0; i < MAXPCHARS; i++) {
 		if ( q == showsyms[i] && (*explainsyms[i])) {
 			if (!found) {
-				pline("%c       %s",q,explainsyms[i]);
-				(void) outspec(explainsyms[i], 0);
+				pline("%c       %s",q,an(explainsyms[i]));
+				(void)outspec(firstmatch = explainsyms[i], 0);
 				found++;
 			}
 			else if (outspec(explainsyms[i], 1))
 				found++;
+			if (i == S_altar || i == S_trap || i == S_web)
+				need_to_print = TRUE;
 		}
 	}
 
 	if (!found)
 		pline("I've never heard of such things.");
+	else if (cc.x != -1) {	/* a specific object on screen */
+		if (found > 1 || need_to_print) {
+			Strcpy(inp, lookat(cc.x, cc.y, q));
+			if (*inp)
+				pline("(%s)", inp);
+		}
+		else {
+			Strcpy(inp, firstmatch);
+		}
+	}
+	else if (found == 1) {
+		Strcpy(inp, firstmatch);
+	}
+	else
+		found = FALSE;	/* abort the 'More info?' stuff */
 
-/* now check for specific things at a given location */
-	if(cc.x != -1 && found) {
-		if(MON_AT(cc.x,cc.y)) {
-			mtmp = m_at(cc.x,cc.y);
-			if (!showmon(mtmp) || Hallucination)
-				mtmp = (struct monst *)0;
-		} else
-			mtmp = (struct monst *) 0;
-		typ = levl[cc.x][cc.y].typ;
-		if (!Invisible 
-#ifdef POLYSELF
-				&& !u.uundetected
-#endif
-				&& u.ux==cc.x && u.uy==cc.y) {
-			pline("(%s named %s)",
-#ifdef POLYSELF
-				u.mtimedone ? mons[u.umonnum].mname :
-#endif
-				pl_character, plname);
-		} else if (mtmp && !mtmp->mimic)
-			pline("(%s%s)",
-			   mtmp->mtame ? "tame " :
-			   mtmp->mpeaceful ? "peaceful " : "",
-			   strncmp(lmonnam(mtmp), "the ", 4)
-				  ? lmonnam(mtmp) : lmonnam(mtmp)+4);
-/* Only worry about the rest of the cases if the symbol could represent
-   more than one thing */
-		else if (found <= 1)
-			/* do nothing */ ;
-		else if (!levl[cc.x][cc.y].seen)
-			pline("(a dark part of a room)");
-#ifdef ALTARS
-		else if (q == showsyms[S_altar] && 
-			 (IS_ALTAR(typ) || (mtmp && mtmp->mimic))) {
-			int kind = levl[cc.x][cc.y].altarmask & ~A_SHRINE;
-			pline( "(%s altar)",
-				(kind == A_CHAOS) ? "chaotic" :
-				(kind == A_NEUTRAL) ? "neutral" :
-				 "lawful" );
-		}
-#endif
-		else if ((q==showsyms[S_ndoor] ||
-			  q==showsyms[S_vodoor] ||
-			  q==showsyms[S_hodoor] ||
-			  q==showsyms[S_cdoor]) &&
-			(IS_DOOR(typ) ||
-				(IS_WALL(typ) && mtmp && mtmp->mimic))) {
-			/* Note: this will say mimics in walls are
-			 *	 closed doors, which we want.
-			 */
-			switch(levl[cc.x][cc.y].doormask & ~D_TRAPPED) {
-				case D_NODOOR: pline("(doorway)"); break;
-				case D_BROKEN: pline("(broken door)"); break;
-				case D_ISOPEN: pline("(open door)"); break;
-				default:       pline("(closed door)"); break;
-						   /* locked or not */
-			}
-		}
-#ifdef STRONGHOLD
-		else if ((q == showsyms[S_dbvwall] ||
-			  q == showsyms[S_dbhwall]) &&
-			  is_db_wall(cc.x,cc.y))
-				pline("(raised drawbridge)");
-#endif
-#ifdef SINKS
-		else if (q == showsyms[S_sink] && IS_SINK(levl[cc.x][cc.y].typ))
-			pline("(sink)");
-#endif
-		else if (IS_ROOM(typ) && q == showsyms[S_room])
-			pline("(floor of a room)");
-		else if (q == showsyms[S_corr] && SPACE_POS(typ))
-			pline("(corridor)");
-		else if (!ACCESSIBLE(typ)) {
-			if (q == showsyms[S_stone] || q == ' ')
-				pline("(dark part of a room)");
+/* check the data file for information about this thing */
+
+checkfile:
+
+	if (!strncmp(inp, "a ", 2))
+		inp += 2;
+	else if (!strncmp(inp, "an ", 3))
+		inp += 3;
+	else if (!strncmp(inp, "the ", 4))
+		inp += 4;
+	if (!strncmp(inp, "tame ", 5))
+		inp += 5;
+	else if (!strncmp(inp, "peaceful ", 9))
+		inp += 9;
+	if (!strncmp(inp, "invisible ", 10))
+		inp += 10;
+
+	if ((!q || found) && *inp) {
+/* adjust the input to remove "named " and convert to lower case */
+ 		for (ep = inp; *ep; ) {
+			if ((!strncmp(ep, " named ", 7) && (alt = ep + 7)) ||
+			    !strncmp(ep, " called ", 8))
+				*ep = 0;
 			else
-				pline("(wall)");
+				(*ep = tolower(*ep)), ep++;
+		}
+
+/*
+ * If the object is named, then the name is the alternate search string;
+ * otherwise, the result of makesingular() applied to the name is. This
+ * isn't strictly optimal, but named objects of interest to the user should
+ * will usually be found under their name, rather than under their
+ * object type, so looking for a singular form is pointless.
+ */
+
+		if (!alt)
+			alt = makesingular(inp);
+		else
+			for (ep = alt; *ep; ep++) *ep = tolower(*ep);
+
+		while(fgets(buf,BUFSZ,fp)) {
+			if(*buf != '\t') {
+			    ep = index(buf, '\n');
+			    if(ep) *ep = 0;
+			    else impossible("bad data file");
+			    if (pmatch(buf, inp)||(alt && pmatch(buf, alt))) {
+				found_in_file = TRUE;
+				break;
+			    }
+			}
 		}
 	}
 
-/* now check for "more info" */
-	if(found_in_file && ep[-1] == ';') {
-		pline("More info? ");
-		if(yn() == 'y') {
-			page_more(fp,1); /* does fclose() */
+	if(found_in_file) {
+/* skip over other possible matches for the info */
+		for(;;) {
+			if ( (i = getc(fp)) == '\t' ) {
+				(void) ungetc(i, fp);
+				break;
+			}
+			if (!fgets(buf, BUFSZ, fp)) {
+				break;
+			}
+		}
+		if (q) {
+			pline("More info? ");
+			if(yn() == 'y') {
+				page_more(fp,1); /* does fclose() */
+				flags.verbose = oldverb;
+				return 0;
+			}
+		}
+		else {
+			page_more(fp, 1);
 			flags.verbose = oldverb;
 			return 0;
 		}
 	}
+	else if (!q)
+		pline("I don't have any information on those things.");
 
 /* if specified by cursor, keep going */
 	if(cc.x != -1) {
-		buf = &bufr[6];
 		more();
 		rewind(fp);
 		goto selobj;
@@ -351,6 +579,17 @@ int strip;	/* nr of chars to be stripped from each line (0 or 1) */
 #if !defined(MSDOS) && !defined(TOS) && !defined(MACOS)
 	int (*prevsig)() = (int (*)())signal(SIGINT, (SIG_RET_TYPE) intruph);
 #endif
+#ifdef MACOS
+	short tmpflags;
+	
+	tmpflags = macflags;
+	macflags &= ~fDoUpdate;
+	if(!mac_more(fp, strip)) {
+		macflags |= (tmpflags & fDoUpdate);
+		return;
+	}
+	macflags |= (tmpflags & fDoUpdate);
+#else
 #if defined(MSDOS) || defined(MINIMAL_TERM)
 	/* There seems to be a bug in ANSI.SYS  The first tab character
 	 * after a clear screen sequence is not expanded correctly.  Thus
@@ -396,11 +635,21 @@ ret:
 	(void) signal(SIGINT, (SIG_RET_TYPE) prevsig);
 	got_intrup = 0;
 #endif
+#endif
 }
 
-static boolean whole_screen = TRUE;
+#endif /* OVLB */
+
 #define	PAGMIN	12	/* minimum # of lines for page below level map */
 
+#ifndef OVLB
+
+OSTATIC boolean whole_screen;
+
+#else /* OVLB */
+
+XSTATIC boolean whole_screen = TRUE;
+
 void
 set_whole_screen() {	/* called in termcap as soon as LI is known */
 	whole_screen = (LI-ROWNO-2 <= PAGMIN || !CD);
@@ -439,7 +688,13 @@ register int mode;	/* 0: open  1: wait+close  2: close */
 		so = flags.standout;
 		flags.standout = 1;
 	} else {
+#ifdef MACOS
+		macflags |= fFullScrKluge;
+#endif
 		if(mode == 1) {
+#ifdef MACOS
+			macflags |= fCornScrKluge;
+#endif
 			curs(1, LI);
 			more();
 		}
@@ -450,12 +705,18 @@ register int mode;	/* 0: open  1: wait+close  2: close */
 			curs(1, ROWNO+4);
 			cl_eos();
 		}
+#ifdef MACOS
+		macflags &= ~fScreenKluges;
+#endif
 	}
 }
 
+#endif /* OVLB */
+#ifdef OVL0
+
 int
 page_line(s)		/* returns 1 if we should quit */
-register char *s;
+register const char *s;
 {
 	if(cury == LI-1) {
 		if(!*s)
@@ -501,7 +762,7 @@ register char *s;
 void
 cornline(mode, text)
 int mode;
-char *text;
+const char *text;
 {
 	static struct line {
 		struct line *next_line;
@@ -556,6 +817,17 @@ char *text;
 	else
 	if(mode == 2) {
 	    register int curline, lth;
+#ifdef MACOS
+		short tmpflags;
+		extern struct line *mactexthead;
+		extern int macmaxlen, maclinect;
+		
+		tmpflags = macflags;
+		macflags |= fDoUpdate | fDisplayKluge;
+		mactexthead = texthead;
+		macmaxlen = maxlen;
+		maclinect = linect;
+#endif
 
 	    if(flags.toplin == 1) more();	/* ab@unido */
 	    remember_topl();
@@ -589,12 +861,21 @@ char *text;
 		curs (lth, curline);
 #endif
 		cl_end ();
-		if (!hmenu) cmore (text);
+		if (!hmenu) {
+#ifdef MACOS
+			macflags |= fCornScrKluge;
+#endif
+			cmore (text);
+		}
 		if (!hmenu || clear_help(hc)) {
 		    home ();
 		    cl_end ();
 		    docorner (lth, curline-1);
 		}
+#ifdef MACOS
+			mactexthead = NULL;
+			macflags |= (tmpflags & (fDoUpdate | fDisplayKluge));
+#endif
 	    } else {					/* feed to pager */
 		set_pager(0);
 		for (tl = texthead; tl; tl = tl->next_line) {
@@ -621,6 +902,9 @@ char *text;
 	}
 }
 
+#endif /* OVL0 */
+#ifdef OVLB
+
 #ifdef WIZARD
 static
 void
@@ -653,21 +937,36 @@ help_menu() {
 	cornline(1, "g.  Longer explanation of game options.");
 	cornline(1, "h.  List of extended commands.");
 	cornline(1, "i.  The NetHack license.");
+#ifdef MACOS
+	cornline(1, "j.  Macintosh primer.");
+#endif
 #ifdef WIZARD
 	if (wizard)
+# ifdef MACOS
+		cornline(1, "k.  List of wizard-mode commands.");
+# else
 		cornline(1, "j.  List of wizard-mode commands.");
+# endif
 #endif
 	cornline(1, "");
 #ifdef WIZARD
 	if (wizard)
+# ifdef MACOS
+		cornline(1, "Select one of a,b,c,d,e,f,g,h,i,j,k or ESC: ");
+# else
 		cornline(1, "Select one of a,b,c,d,e,f,g,h,i,j or ESC: ");
+# endif
 	else
 #endif
+#ifdef MACOS
+		cornline(1, "Select one of a,b,c,d,e,f,g,h,i,j or ESC: ");
+#else
 		cornline(1, "Select one of a,b,c,d,e,f,g,h,i or ESC: ");
+#endif
 	cornline(-1,"");
 }
 
-static boolean
+XSTATIC boolean
 clear_help(c)
 char c;
 {
@@ -696,19 +995,31 @@ char c;
 	        || !whole_screen
 #endif
 #ifdef WIZARD
+# ifdef MACOS
+		|| c == 'k'
+# else
 		|| c == 'j'
+# endif
 #endif
 		);
 }
 
-static boolean
+XSTATIC boolean
 valid_help(c)
 char c;
 {
 #ifdef WIZARD
+# ifdef MACOS
+	return ((c >= 'a' && c <= (wizard ? 'k' : 'j')) || index(quitchars,c));
+# else
 	return ((c >= 'a' && c <= (wizard ? 'j' : 'i')) || index(quitchars,c));
+# endif
 #else
+# ifdef MACOS
+	return ((c >= 'a' && c <= 'j') || index(quitchars,c));
+# else
 	return ((c >= 'a' && c <= 'i') || index(quitchars,c));
+# endif
 #endif
 }
 
@@ -717,8 +1028,8 @@ dohelp()
 {
 #ifdef MACOS
 	term_info	*t;
-	
-	macflags &= ~(fDoUpdate | fDoNonKeyEvt);
+
+	macflags &= ~fDoNonKeyEvt;
 	t = (term_info *)GetWRefCon(HackWindow);
 	SetVol((StringPtr)NULL,
 		(t->auxFileVRefNum) ? t->auxFileVRefNum : t->recordVRefNum);
@@ -736,13 +1047,18 @@ dohelp()
 			case 'h':  (void) doextlist();  break;
 			case 'i':  (void) page_file(LICENSE, FALSE);  break;
 #ifdef WIZARD
+# ifdef MACOS
+			case 'j':  (void) page_file(MACHELP, FALSE);  break;
+			case 'k':  wiz_help();  break;
+# else
 			case 'j':  wiz_help();  break;
+# endif
 #endif
 		}
 	}
 #ifdef MACOS
-	macflags |= (fDoUpdate | fDoNonKeyEvt);
 	SetVol((StringPtr)NULL, t->recordVRefNum);
+	macflags |= fDoNonKeyEvt;
 #endif
 	return 0;
 }
@@ -756,7 +1072,7 @@ dohistory()
 
 int
 page_file(fnam, silent)	/* return: 0 - cannot open fnam; 1 - otherwise */
-register char *fnam;
+register const char *fnam;
 boolean silent;
 {
 #ifdef DEF_PAGER			/* this implies that UNIX is defined */
@@ -798,12 +1114,7 @@ boolean silent;
 # ifdef MACOS
 	if ((f = fopen (fnam, "r")) == (FILE *) 0)
 		f = openFile(fnam, "r");
-	/* refresh screen kluge */
 	if (!f) {
-		cls();
-		docrt();
-		clrlin();
-		ValidRect(&(**(*HackWindow).visRgn).rgnBBox);
 # else
 	if ((f = fopen (fnam, "r")) == (FILE *) 0) {
 # endif
@@ -887,3 +1198,5 @@ register int f = fork();
 }
 #endif
 #endif /* UNIX /**/
+
+#endif /* OVLB */
